@@ -1,12 +1,20 @@
-# -------------------------------------------------------------------------
-# Copyright (c) Microsoft Corporation. All rights reserved.
-# Licensed under the MIT License. See License.txt in the project root for
-# license information.
-# --------------------------------------------------------------------------
-from ..common._common_conversion import _to_str
-
-
+﻿#-------------------------------------------------------------------------
+# Copyright (c) Microsoft.  All rights reserved.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#--------------------------------------------------------------------------
+from .._common_conversion import _to_str
 class Container(object):
+
     '''
     Blob container class. 
     
@@ -17,7 +25,7 @@ class Container(object):
         This var is set to None unless the include=metadata param was included 
         for the list containers operation. If this parameter was specified but the 
         container has no metadata, metadata will be set to an empty dictionary.
-    :vartype metadata: dict(str, str)
+    :vartype metadata: dict mapping str to str
     :ivar ContainerProperties properties:
         System properties for the container.
     '''
@@ -29,6 +37,7 @@ class Container(object):
 
 
 class ContainerProperties(object):
+
     '''
     Blob container's properties class.
     
@@ -49,6 +58,7 @@ class ContainerProperties(object):
 
 
 class Blob(object):
+
     '''
     Blob class.
     
@@ -65,23 +75,17 @@ class Blob(object):
         Stores all the system properties for the blob.
     :ivar metadata:
         Name-value pairs associated with the blob as metadata.
-    :ivar bool deleted:
-        Specify whether the blob was soft deleted.
-        In other words, if the blob is being retained by the delete retention policy,
-        this field would be True. The blob could be undeleted or it will be garbage collected after the specified
-        time period.
     '''
-
-    def __init__(self, name=None, snapshot=None, content=None, props=None, metadata=None, deleted=False):
+    def __init__(self, name=None, snapshot=None, content=None, props=None, metadata=None):
         self.name = name
         self.snapshot = snapshot
         self.content = content
         self.properties = props or BlobProperties()
         self.metadata = metadata
-        self.deleted = deleted
 
 
 class BlobProperties(object):
+
     '''
     Blob Properties
     
@@ -112,22 +116,6 @@ class BlobProperties(object):
         Stores all the content settings for the blob.
     :ivar ~azure.storage.blob.models.LeaseProperties lease:
         Stores all the lease information for the blob.
-    :ivar StandardBlobTier blob_tier:
-        Indicates the access tier of the blob. The hot tier is optimized
-        for storing data that is accessed frequently. The cool storage tier
-        is optimized for storing data that is infrequently accessed and stored
-        for at least a month. The archive tier is optimized for storing
-        data that is rarely accessed and stored for at least six months
-        with flexible latency requirements.
-    :ivar datetime blob_tier_change_time:
-        Indicates when the access tier was last changed.
-    :ivar bool blob_tier_inferred:
-        Indicates whether the access tier was inferred by the service.
-        If false, it indicates that the tier was set explicitly.
-    :ivar datetime deleted_time:
-        A datetime object representing the time at which the blob was deleted.
-    :ivar int remaining_retention_days:
-        The number of days that the blob will be retained before being permanently deleted by the service.
     '''
 
     def __init__(self):
@@ -142,14 +130,10 @@ class BlobProperties(object):
         self.copy = CopyProperties()
         self.content_settings = ContentSettings()
         self.lease = LeaseProperties()
-        self.blob_tier = None
-        self.blob_tier_change_time = None
-        self.blob_tier_inferred = False
-        self.deleted_time = None
-        self.remaining_retention_days = None
 
 
 class ContentSettings(object):
+
     '''
     Used to store the content settings of a blob.
     
@@ -177,9 +161,10 @@ class ContentSettings(object):
     '''
 
     def __init__(
-            self, content_type=None, content_encoding=None,
-            content_language=None, content_disposition=None,
-            cache_control=None, content_md5=None):
+        self, content_type=None, content_encoding=None,
+        content_language=None, content_disposition=None,
+        cache_control=None, content_md5=None):
+        
         self.content_type = content_type
         self.content_encoding = content_encoding
         self.content_language = content_language
@@ -248,15 +233,15 @@ class CopyProperties(object):
 
 
 class LeaseProperties(object):
+
     '''
     Blob Lease Properties.
     
     :ivar str status:
         The lease status of the blob.
-        Possible values: locked|unlocked
     :ivar str state:
         Lease state of the blob.
-        Possible values: available|leased|expired|breaking|broken
+        Possible values: pending|success|aborted|failed
     :ivar str duration:
         When a blob is leased, specifies whether the lease is of infinite or fixed duration.
     '''
@@ -294,6 +279,7 @@ class BlobBlockState(object):
 
 
 class BlobBlock(object):
+
     '''
     BlockBlob Block class.
     
@@ -315,23 +301,24 @@ class BlobBlock(object):
 
 
 class BlobBlockList(object):
+
     '''
     Blob Block List class.
    
     :ivar committed_blocks:
         List of committed blocks.
-    :vartype committed_blocks: list(:class:`~azure.storage.blob.models.BlobBlock`)
+    :vartype committed_blocks: list of :class:`BlobBlock`
     :ivar uncommitted_blocks:
         List of uncommitted blocks.
-    :vartype uncommitted_blocks: list(:class:`~azure.storage.blob.models.BlobBlock`)
+    :vartype uncommitted_blocks: list of :class:`BlobBlock`
     '''
 
     def __init__(self):
         self.committed_blocks = list()
         self.uncommitted_blocks = list()
 
-
 class PageRange(object):
+
     '''
     Page Range for page blob.
     
@@ -349,8 +336,8 @@ class PageRange(object):
         self.end = end
         self.is_cleared = is_cleared
 
-
 class ResourceProperties(object):
+
     '''
     Base response for a resource request.
     
@@ -365,8 +352,8 @@ class ResourceProperties(object):
         self.last_modified = None
         self.etag = None
 
-
 class AppendBlockProperties(ResourceProperties):
+
     '''
     Response for an append block request.
     
@@ -383,6 +370,7 @@ class AppendBlockProperties(ResourceProperties):
 
 
 class PageBlobProperties(ResourceProperties):
+
     '''
     Response for a page request.
     
@@ -420,7 +408,6 @@ class PublicAccess(object):
     within the storage account.
     '''
 
-
 class DeleteSnapshot(object):
     '''
     Required if the blob has associated snapshots. Specifies how to handle the snapshots.
@@ -436,7 +423,6 @@ class DeleteSnapshot(object):
     Delete only the blob's snapshots and not the blob itself.
     '''
 
-
 class BlockListType(object):
     '''
     Specifies whether to return the list of committed blocks, the list of uncommitted 
@@ -490,7 +476,6 @@ class _LeaseActions(object):
     Renew = 'renew'
     '''Renew the lease.'''
 
-
 class _BlobTypes(object):
     '''Blob type options.'''
 
@@ -503,8 +488,8 @@ class _BlobTypes(object):
     PageBlob = 'PageBlob'
     '''Page blob type.'''
 
-
 class Include(object):
+
     '''
     Specifies the datasets to include in the blob list response.
 
@@ -518,12 +503,10 @@ class Include(object):
     :ivar ~azure.storage.blob.models.Include Include.UNCOMMITTED_BLOBS: 
         Specifies that blobs for which blocks have been uploaded, but which have not 
         been committed using Put Block List, be included in the response.
-    :ivar ~azure.storage.blob.models.Include Include.DELETED:
-        Specifies that deleted blobs should be returned in the response.
     '''
 
-    def __init__(self, snapshots=False, metadata=False, uncommitted_blobs=False,
-                 copy=False, deleted=False, _str=None):
+    def __init__(self, snapshots=False, metadata=False, uncommitted_blobs=False, 
+                 copy=False, _str=None):
         '''
         :param bool snapshots:
              Specifies that snapshots should be included in the enumeration.
@@ -534,9 +517,7 @@ class Include(object):
             not been committed using Put Block List, be included in the response.
         :param bool copy: 
             Specifies that metadata related to any current or previous Copy Blob 
-            operation should be included in the response.
-        :param bool deleted:
-            Specifies that deleted blobs should be returned in the response.
+            operation should be included in the response. 
         :param str _str: 
             A string representing the includes.
         '''
@@ -547,31 +528,28 @@ class Include(object):
         self.metadata = metadata or ('metadata' in components)
         self.uncommitted_blobs = uncommitted_blobs or ('uncommittedblobs' in components)
         self.copy = copy or ('copy' in components)
-        self.deleted = deleted or ('deleted' in components)
-
+    
     def __or__(self, other):
         return Include(_str=str(self) + str(other))
 
     def __add__(self, other):
         return Include(_str=str(self) + str(other))
-
+    
     def __str__(self):
-        include = (('snapshots,' if self.snapshots else '') +
+        include = (('snapshots,' if self.snapshots else '') + 
                    ('metadata,' if self.metadata else '') +
                    ('uncommittedblobs,' if self.uncommitted_blobs else '') +
-                   ('copy,' if self.copy else '') +
-                   ('deleted,' if self.deleted else ''))
+                   ('copy,' if self.copy else ''))
         return include.rstrip(',')
 
-
 Include.COPY = Include(copy=True)
 Include.METADATA = Include(metadata=True)
 Include.SNAPSHOTS = Include(snapshots=True)
 Include.UNCOMMITTED_BLOBS = Include(uncommitted_blobs=True)
-Include.DELETED = Include(deleted=True)
 
 
 class BlobPermissions(object):
+
     '''
     BlobPermissions class to be used with 
     :func:`~azure.storage.blob.baseblobservice.BaseBlobService.generate_blob_shared_access_signature` API.
@@ -590,7 +568,7 @@ class BlobPermissions(object):
         copy operation within the same account.
     '''
 
-    def __init__(self, read=False, add=False, create=False, write=False,
+    def __init__(self, read=False, add=False, create=False, write=False, 
                  delete=False, _str=None):
         '''    
         :param bool read:
@@ -616,13 +594,13 @@ class BlobPermissions(object):
         self.create = create or ('c' in _str)
         self.write = write or ('w' in _str)
         self.delete = delete or ('d' in _str)
-
+    
     def __or__(self, other):
         return BlobPermissions(_str=str(self) + str(other))
 
     def __add__(self, other):
         return BlobPermissions(_str=str(self) + str(other))
-
+    
     def __str__(self):
         return (('r' if self.read else '') +
                 ('a' if self.add else '') +
@@ -630,7 +608,6 @@ class BlobPermissions(object):
                 ('w' if self.write else '') +
                 ('d' if self.delete else ''))
 
-
 BlobPermissions.ADD = BlobPermissions(add=True)
 BlobPermissions.CREATE = BlobPermissions(create=True)
 BlobPermissions.DELETE = BlobPermissions(delete=True)
@@ -639,6 +616,7 @@ BlobPermissions.WRITE = BlobPermissions(write=True)
 
 
 class ContainerPermissions(object):
+
     '''
     ContainerPermissions class to be used with :func:`~azure.storage.blob.baseblobservice.BaseBlobService.generate_container_shared_access_signature`
     API and for the AccessPolicies used with :func:`~azure.storage.blob.baseblobservice.BaseBlobService.set_container_acl`. 
@@ -660,7 +638,7 @@ class ContainerPermissions(object):
         a container SAS. Use an account SAS instead.
     '''
 
-    def __init__(self, read=False, write=False, delete=False, list=False,
+    def __init__(self, read=False, write=False, delete=False, list=False, 
                  _str=None):
         '''
         :param bool read:
@@ -687,69 +665,20 @@ class ContainerPermissions(object):
         self.write = write or ('w' in _str)
         self.delete = delete or ('d' in _str)
         self.list = list or ('l' in _str)
-
+    
     def __or__(self, other):
         return ContainerPermissions(_str=str(self) + str(other))
 
     def __add__(self, other):
         return ContainerPermissions(_str=str(self) + str(other))
-
+    
     def __str__(self):
         return (('r' if self.read else '') +
                 ('w' if self.write else '') +
-                ('d' if self.delete else '') +
+                ('d' if self.delete else '') + 
                 ('l' if self.list else ''))
 
-
 ContainerPermissions.DELETE = ContainerPermissions(delete=True)
 ContainerPermissions.LIST = ContainerPermissions(list=True)
 ContainerPermissions.READ = ContainerPermissions(read=True)
 ContainerPermissions.WRITE = ContainerPermissions(write=True)
-
-
-class PremiumPageBlobTier(object):
-    '''
-    Specifies the page blob tier to set the blob to. This is only applicable to page
-    blobs on premium storage accounts.
-    Please take a look at https://docs.microsoft.com/en-us/azure/storage/storage-premium-storage#scalability-and-performance-targets
-    for detailed information on the corresponding IOPS and throughtput per PageBlobTier.
-    '''
-
-    P4 = 'P4'
-    ''' P4 Tier '''
-
-    P6 = 'P6'
-    ''' P6 Tier '''
-
-    P10 = 'P10'
-    ''' P10 Tier '''
-
-    P20 = 'P20'
-    ''' P20 Tier '''
-
-    P30 = 'P30'
-    ''' P30 Tier '''
-
-    P40 = 'P40'
-    ''' P40 Tier '''
-
-    P50 = 'P50'
-    ''' P50 Tier '''
-
-    P60 = 'P60'
-    ''' P60 Tier '''
-
-
-class StandardBlobTier(object):
-    '''
-    Specifies the blob tier to set the blob to. This is only applicable for block blobs on standard storage accounts.
-    '''
-
-    Archive = 'Archive'
-    ''' Archive '''
-
-    Cool = 'Cool'
-    ''' Cool '''
-
-    Hot = 'Hot'
-    ''' Hot '''
