@@ -14,13 +14,28 @@
 #--------------------------------------------------------------------------
 import sys
 if sys.version_info < (3,):
+    from collections import Iterable
     _unicode_type = unicode
 else:
+    from collections.abc import Iterable
     _unicode_type = str
 
 from ._error import (
     _validate_not_none,
+    _ERROR_UNKNOWN_KEY_WRAP_ALGORITHM
 )
+from cryptography.hazmat.primitives.keywrap import(
+    aes_key_wrap,
+    aes_key_unwrap,
+)
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives.asymmetric.rsa import generate_private_key
+from cryptography.hazmat.primitives.asymmetric.padding import (
+    OAEP,
+    MGF1,
+)
+from cryptography.hazmat.primitives.hashes import SHA1
+from os import urandom
 
 class _HeaderDict(dict):
 
@@ -35,7 +50,21 @@ class _dict(dict):
     '''Used so that additional properties can be set on the return dictionary'''
     pass
 
-class ListGenerator(object):
+class _OperationContext(object):
+    '''
+    Contains information that lasts the lifetime of an operation. This operation 
+    may span multiple calls to the Azure service.
+
+    :ivar bool location_lock: 
+        Whether the location should be locked for this operation.
+    :ivar str location: 
+        The location to lock to.
+    '''
+    def __init__(self, location_lock=False):
+        self.location_lock = location_lock
+        self.host_location = None
+
+class ListGenerator(Iterable):
     '''
     A generator object used to list storage resources. The generator will lazily 
     follow the continuation tokens returned by the service and stop when all 
@@ -88,6 +117,39 @@ class ListGenerator(object):
             for i in self.items:
                 yield i
 
+
+class RetryContext(object):
+    '''
+    Contains the request and response information that can be used to determine 
+    whether and how to retry. This context is stored across retries and may be 
+    used to store other information relevant to the retry strategy.
+
+    :ivar :class:`~azure.storage._http.HTTPRequest` request: 
+        The request sent to the storage service.
+    :ivar :class:`~azure.storage._http.HTTPResponse` response: 
+        The response returned by the storage service.
+    :ivar LocationMode location_mode: 
+        The location the request was sent to.
+    '''
+    def __init__(self):
+        self.request = None
+        self.response = None
+        self.location_mode = None
+
+class LocationMode(object):
+    '''
+    Specifies the location the request should be sent to. This mode only applies 
+    for RA-GRS accounts which allow secondary read access. All other account types 
+    must use PRIMARY.
+    '''
+
+    PRIMARY = 'primary'
+    ''' Requests should be sent to the primary location. '''
+
+    SECONDARY = 'secondary'
+    ''' Requests should be sent to the secondary location, if possible. '''
+
+
 class RetentionPolicy(object):
 
     '''
@@ -272,6 +334,42 @@ class ServiceProperties(object):
     pass
 
 
+class ServiceStats(object):
+    ''' 
+    Returned by get_*_service_stats functions. Contains statistics related to 
+    replication for the given service. It is only available when read-access 
+    geo-redundant replication is enabled for the storage account.
+
+    :ivar GeoReplication geo_replication:
+        An object containing statistics related to replication for the given service.
+    '''
+    pass
+
+
+class GeoReplication(object):
+    ''' 
+    Contains statistics related to replication for the given service.
+
+    :ivar str status:
+        The status of the secondary location. Possible values are:
+            live: Indicates that the secondary location is active and operational.
+            bootstrap: Indicates initial synchronization from the primary location 
+                to the secondary location is in progress. This typically occurs 
+                when replication is first enabled.
+            unavailable: Indicates that the secondary location is temporarily 
+                unavailable.
+    :ivar date last_sync_time:
+        A GMT date value, to the second. All primary writes preceding this value 
+        are guaranteed to be available for read operations at the secondary. 
+        Primary writes after this point in time may or may not be available for 
+        reads. The value may be empty if LastSyncTime is not available. This can 
+        happen if the replication status is bootstrap or unavailable. Although 
+        geo-replication is continuously enabled, the LastSyncTime result may 
+        reflect a cached value from the service that is refreshed every few minutes.
+    '''
+    pass
+
+
 class AccessPolicy(object):
 
     ''' 
@@ -336,7 +434,6 @@ class Protocol(object):
     HTTPS_HTTP = 'https,http'
     ''' Allow HTTP and HTTPS requests. '''
 
-
 class ResourceTypes(object):
 
     '''
@@ -532,4 +629,4 @@ AccountPermissions.LIST = AccountPermissions(list=True)
 AccountPermissions.ADD = AccountPermissions(add=True)
 AccountPermissions.CREATE = AccountPermissions(create=True)
 AccountPermissions.UPDATE = AccountPermissions(update=True)
-AccountPermissions.PROCESS = AccountPermissions(process=True)
+AccountPermissions.PROCESS = AccountPermissions(process=True)
\ No newline at end of file
