@@ -1,4 +1,4 @@
-﻿#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 # Copyright (c) Microsoft.  All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,8 +11,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#--------------------------------------------------------------------------
+# --------------------------------------------------------------------------
 import sys
+
 if sys.version_info >= (3,):
     from io import BytesIO
 else:
@@ -26,10 +27,13 @@ try:
 except ImportError:
     from xml.etree import ElementTree as ETree
 
-from xml.sax.saxutils import escape as xml_escape
-from .._common_conversion import (
+from ..common._common_conversion import (
     _str,
 )
+from ._encryption import (
+    _encrypt_queue_message,
+)
+
 
 def _get_path(queue_name=None, include_messages=None, message_id=None):
     '''
@@ -52,17 +56,19 @@ def _get_path(queue_name=None, include_messages=None, message_id=None):
         return '/'
 
 
-def _convert_queue_message_xml(message_text, encode_function):
+def _convert_queue_message_xml(message_text, encode_function, key_encryption_key):
     '''
     <?xml version="1.0" encoding="utf-8"?>
     <QueueMessage>
         <MessageText></MessageText>
     </QueueMessage>
     '''
-    queue_message_element = ETree.Element('QueueMessage');
+    queue_message_element = ETree.Element('QueueMessage')
 
     # Enabled
     message_text = encode_function(message_text)
+    if key_encryption_key is not None:
+        message_text = _encrypt_queue_message(message_text, key_encryption_key)
     ETree.SubElement(queue_message_element, 'MessageText').text = message_text
 
     # Add xml declaration and serialize
