@@ -1,16 +1,7 @@
 # -------------------------------------------------------------------------
-# Copyright (c) Microsoft.  All rights reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#   http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# Copyright (c) Microsoft Corporation. All rights reserved.
+# Licensed under the MIT License. See License.txt in the project root for
+# license information.
 # --------------------------------------------------------------------------
 from azure.common import AzureException
 from dateutil import parser
@@ -26,7 +17,7 @@ from ..common._common_conversion import (
 )
 from ..common._deserialization import (
     _parse_properties,
-    _int_to_str,
+    _to_int,
     _parse_metadata,
     _convert_xml_to_signed_identifiers,
     _bool,
@@ -71,7 +62,7 @@ def _parse_page_properties(response):
     put_page = PageBlobProperties()
     put_page.last_modified = parser.parse(response.headers.get('last-modified'))
     put_page.etag = response.headers.get('etag')
-    put_page.sequence_number = _int_to_str(response.headers.get('x-ms-blob-sequence-number'))
+    put_page.sequence_number = _to_int(response.headers.get('x-ms-blob-sequence-number'))
 
     return put_page
 
@@ -83,8 +74,8 @@ def _parse_append_block(response):
     append_block = AppendBlockProperties()
     append_block.last_modified = parser.parse(response.headers.get('last-modified'))
     append_block.etag = response.headers.get('etag')
-    append_block.append_offset = _int_to_str(response.headers.get('x-ms-blob-append-offset'))
-    append_block.committed_block_count = _int_to_str(response.headers.get('x-ms-blob-committed-block-count'))
+    append_block.append_offset = _to_int(response.headers.get('x-ms-blob-append-offset'))
+    append_block.committed_block_count = _to_int(response.headers.get('x-ms-blob-committed-block-count'))
 
     return append_block
 
@@ -104,7 +95,7 @@ def _parse_lease(response):
     '''
     lease = {'time': response.headers.get('x-ms-lease-time')}
     if lease['time']:
-        lease['time'] = _int_to_str(lease['time'])
+        lease['time'] = _to_int(lease['time'])
 
     lease['id'] = response.headers.get('x-ms-lease-id')
 
@@ -224,9 +215,9 @@ def _convert_xml_to_containers(response):
 LIST_BLOBS_ATTRIBUTE_MAP = {
     'Last-Modified': (None, 'last_modified', parser.parse),
     'Etag': (None, 'etag', _to_str),
-    'x-ms-blob-sequence-number': (None, 'sequence_number', _int_to_str),
+    'x-ms-blob-sequence-number': (None, 'sequence_number', _to_int),
     'BlobType': (None, 'blob_type', _to_str),
-    'Content-Length': (None, 'content_length', _int_to_str),
+    'Content-Length': (None, 'content_length', _to_int),
     'ServerEncrypted': (None, 'server_encrypted', _bool),
     'Content-Type': ('content_settings', 'content_type', _to_str),
     'Content-Encoding': ('content_settings', 'content_encoding', _to_str),
@@ -247,6 +238,8 @@ LIST_BLOBS_ATTRIBUTE_MAP = {
     'AccessTierChangeTime': (None, 'blob_tier_change_time', parser.parse),
     'AccessTierInferred': (None, 'blob_tier_inferred', _bool),
     'ArchiveStatus': (None, 'rehydration_status', _to_str),
+    'DeletedTime': (None, 'deleted_time', parser.parse),
+    'RemainingRetentionDays': (None, 'remaining_retention_days', _to_int),
 }
 
 
@@ -261,6 +254,7 @@ def _convert_xml_to_blob_list(response):
       <Blobs>
         <Blob>
           <Name>blob-name</name>
+          <Deleted>true</Deleted>
           <Snapshot>date-time-value</Snapshot>
           <Properties>
             <Last-Modified>date-time-value</Last-Modified>
@@ -285,6 +279,8 @@ def _convert_xml_to_blob_list(response):
             <AccessTier>P4 | P6 | P10 | P20 | P30 | P40 | P50 | P60 | Archive | Cool | Hot</AccessTier>
             <AccessTierChangeTime>date-time-value</AccessTierChangeTime>
             <AccessTierInferred>true</AccessTierInferred>
+            <DeletedTime>datetime</DeletedTime>
+            <RemainingRetentionDays>int</RemainingRetentionDays>
           </Properties>
           <Metadata>   
             <Name>value</Name>
@@ -318,6 +314,10 @@ def _convert_xml_to_blob_list(response):
         blob.name = blob_element.findtext('Name')
         blob.snapshot = blob_element.findtext('Snapshot')
 
+        deleted = blob_element.findtext('Deleted')
+        if deleted:
+            blob.deleted = _bool(deleted)
+
         # Properties
         properties_element = blob_element.find('Properties')
         if properties_element is not None:
