@@ -14,7 +14,6 @@ from time import sleep
 import requests
 from azure.common import (
     AzureException,
-    AzureHttpError,
 )
 
 from ._constants import (
@@ -70,10 +69,10 @@ class StorageClient(object):
         reading from secondary.
     :ivar function(context) retry:
         A function which determines whether to retry. Takes as a parameter a 
-        :class:`~.models.RetryContext` object. Returns the number
+        :class:`~azure.storage.common.models.RetryContext` object. Returns the number
         of seconds to wait before retrying the request, or None to indicate not 
         to retry.
-    :ivar ~.models.LocationMode location_mode:
+    :ivar ~azure.storage.common.models.LocationMode location_mode:
         The host location to use to make requests. Defaults to LocationMode.PRIMARY.
         Note that this setting only applies to RA-GRS accounts as other account 
         types do not allow reading from secondary. If the location_mode is set to 
@@ -210,7 +209,7 @@ class StorageClient(object):
         else:
             return ""
 
-    def _perform_request(self, request, parser=None, parser_args=None, operation_context=None, expected_errors=None):
+    def _perform_request(self, request, parser=None, parser_args=None, operation_context=None):
         '''
         Sends the request and return response. Catches HTTPError and hands it
         to error handler
@@ -282,7 +281,7 @@ class StorageClient(object):
                                 self.extract_date_and_request_id(retry_context),
                                 response.status,
                                 response.message,
-                                str(response.headers).replace('\n', ''))
+                                str(request.headers).replace('\n', ''))
 
                     # Parse and wrap HTTP errors in AzureHttpError which inherits from AzureException
                     if response.status >= 300:
@@ -326,16 +325,6 @@ class StorageClient(object):
                     status_code = retry_context.response.status if retry_context.response is not None else 'Unknown'
                     timestamp_and_request_id = self.extract_date_and_request_id(retry_context)
 
-                # if the http error was expected, we should short-circuit
-                if isinstance(ex, AzureHttpError) and expected_errors is not None and ex.error_code in expected_errors:
-                    logger.info("%s Received expected http error: "
-                                "%s, HTTP status code=%s, Exception=%s.",
-                                client_request_id_prefix,
-                                timestamp_and_request_id,
-                                status_code,
-                                exception_str_in_one_line)
-                    raise ex
-
                 logger.info("%s Operation failed: checking if the operation should be retried. "
                             "Current retry count=%s, %s, HTTP status code=%s, Exception=%s.",
                             client_request_id_prefix,
@@ -387,5 +376,4 @@ class StorageClient(object):
                     # note: to cover the emulator scenario, the host_location is grabbed
                     # from request.host_locations(which includes the dev account name)
                     # instead of request.host(which at this point no longer includes the dev account name)
-                    operation_context.host_location = {
-                        retry_context.location_mode: request.host_locations[retry_context.location_mode]}
+                    operation_context.host_location = {retry_context.location_mode: request.host_locations[retry_context.location_mode]}
