@@ -1,16 +1,7 @@
 # -------------------------------------------------------------------------
-# Copyright (c) Microsoft.  All rights reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#   http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+# Copyright (c) Microsoft Corporation. All rights reserved.
+# Licensed under the MIT License. See License.txt in the project root for
+# license information.
 # --------------------------------------------------------------------------
 import sys
 
@@ -120,15 +111,19 @@ class RetryContext(object):
     whether and how to retry. This context is stored across retries and may be 
     used to store other information relevant to the retry strategy.
 
-    :ivar ~azure.storage.common._http.HTTPRequest request:
+    :ivar ~._http.HTTPRequest request:
         The request sent to the storage service.
-    :ivar ~azure.storage.common._http.HTTPResponse response:
+    :ivar ~._http.HTTPResponse response:
         The response returned by the storage service.
     :ivar LocationMode location_mode:
         The location the request was sent to.
     :ivar Exception exception:
         The exception that just occurred. The type could either be AzureException (for HTTP errors),
         or other Exception types from lower layers, which are kept unwrapped for easier processing.
+    :ivar bool is_emulated:
+        Whether retry is targeting the emulator. The default value is False.
+    :ivar int body_position:
+        The initial position of the body stream. It is useful when retries happen and we need to rewind the stream.
     '''
 
     def __init__(self):
@@ -136,6 +131,8 @@ class RetryContext(object):
         self.response = None
         self.location_mode = None
         self.exception = None
+        self.is_emulated = False
+        self.body_position = None
 
 
 class LocationMode(object):
@@ -308,6 +305,52 @@ class CorsRule(object):
         self.allowed_headers = allowed_headers if allowed_headers else list()
 
 
+class DeleteRetentionPolicy(object):
+    '''
+    To set DeleteRetentionPolicy, you must call Set Blob Service Properties using version 2017-07-29 or later.
+    This class groups the settings related to delete retention policy.
+    '''
+
+    def __init__(self, enabled=False, days=None):
+        '''
+        :param bool enabled:
+            Required. Indicates whether a deleted blob or snapshot is retained or immediately removed by delete operation.
+        :param int days:
+            Required only if Enabled is true. Indicates the number of days that deleted blob be retained.
+            All data older than this value will be permanently deleted.
+            The minimum value you can specify is 1; the largest value is 365.
+        '''
+        _validate_not_none("enabled", enabled)
+        if enabled:
+            _validate_not_none("days", days)
+
+        self.enabled = enabled
+        self.days = days
+
+
+class StaticWebsite(object):
+    '''
+    Class representing the service properties pertaining to static websites.
+    To set StaticWebsite, you must call Set Blob Service Properties using version 2018-03-28 or later.
+    '''
+
+    def __init__(self, enabled=False, index_document=None, error_document_404_path=None):
+        '''
+        :param bool enabled:
+            Required. True if static websites should be enabled on the blob service for the corresponding Storage Account.
+        :param str index_document:
+            Represents the name of the index document. This is commonly "index.html".
+        :param str error_document_404_path:
+            Represents the path to the error document that should be shown when an error 404 is issued,
+            in other words, when a browser requests a page that does not exist.
+        '''
+        _validate_not_none("enabled", enabled)
+
+        self.enabled = enabled
+        self.index_document = index_document
+        self.error_document_404_path = error_document_404_path
+
+
 class ServiceProperties(object):
     ''' 
     Returned by get_*_service_properties functions. Contains the properties of a 
@@ -493,9 +536,10 @@ class Services(object):
     :ivar Services Services.BLOB: The blob service.
     :ivar Services Services.FILE: The file service
     :ivar Services Services.QUEUE: The queue service.
+    :ivar Services Services.TABLE: The table service.
     '''
 
-    def __init__(self, blob=False, queue=False, file=False, _str=None):
+    def __init__(self, blob=False, queue=False, file=False, table=False, _str=None):
         '''
         :param bool blob:
             Access to any blob service, for example, the `.BlockBlobService`
@@ -503,6 +547,8 @@ class Services(object):
             Access to the `.QueueService`
         :param bool file:
             Access to the `.FileService`
+        :param bool table:
+            Access to the TableService
         :param str _str: 
             A string representing the services.
         '''
@@ -511,6 +557,7 @@ class Services(object):
         self.blob = blob or ('b' in _str)
         self.queue = queue or ('q' in _str)
         self.file = file or ('f' in _str)
+        self.table = table or ('t' in _str)
 
     def __or__(self, other):
         return Services(_str=str(self) + str(other))
@@ -521,11 +568,13 @@ class Services(object):
     def __str__(self):
         return (('b' if self.blob else '') +
                 ('q' if self.queue else '') +
+                ('t' if self.table else '') +
                 ('f' if self.file else ''))
 
 
 Services.BLOB = Services(blob=True)
 Services.QUEUE = Services(queue=True)
+Services.TABLE = Services(table=True)
 Services.FILE = Services(file=True)
 
 
