@@ -3,19 +3,12 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-# pylint: disable=invalid-overridden-method
-from typing import Any
-try:
-    from urllib.parse import quote, unquote
-except ImportError:
-    from urllib2 import quote, unquote # type: ignore
 
-from azure.core.exceptions import HttpResponseError
 from ._download_async import StorageStreamDownloader
 from ._path_client_async import PathClient
 from .._data_lake_file_client import DataLakeFileClient as DataLakeFileClientBase
-from .._serialize import convert_datetime_to_rfc1123
-from .._deserialize import process_storage_error, deserialize_file_properties
+from .._deserialize import process_storage_error
+from .._generated.models import StorageErrorException
 from .._models import FileProperties
 from ..aio._upload_helper import upload_datalake_file
 
@@ -40,11 +33,9 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
     :type file_path: str
     :param credential:
         The credentials with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string,
-        an instance of a AzureSasCredential from azure.core.credentials, an account
+        account URL already has a SAS token. The value can be a SAS token string, and account
         shared access key, or an instance of a TokenCredentials class from azure.identity.
-        If the resource URI already contains a SAS token, this will be ignored in favor of an explicit credential
-        - except in the case of AzureSasCredential, where the conflicting SAS tokens will raise a ValueError.
+        If the URL already has a SAS token, specifying an explicit credential will take priority.
 
     .. admonition:: Example:
 
@@ -130,17 +121,6 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
         """
         return await self._create('file', content_settings=content_settings, metadata=metadata, **kwargs)
 
-    async def exists(self, **kwargs):
-        # type: (**Any) -> bool
-        """
-        Returns True if a file exists and returns False otherwise.
-
-        :kwarg int timeout:
-            The timeout parameter is expressed in seconds.
-        :returns: boolean
-        """
-        return await self._exists(**kwargs)
-
     async def delete_file(self, **kwargs):
         # type: (...) -> None
         """
@@ -221,30 +201,8 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
                 :dedent: 4
                 :caption: Getting the properties for a file.
         """
-        return await self._get_path_properties(cls=deserialize_file_properties, **kwargs)  # pylint: disable=protected-access
-
-    async def set_file_expiry(self, expiry_options,  # type: str
-                              expires_on=None,  # type: Optional[Union[datetime, int]]
-                              **kwargs):
-        # type: (str, Optional[Union[datetime, int]], **Any) -> None
-        """Sets the time a file will expire and be deleted.
-
-        :param str expiry_options:
-            Required. Indicates mode of the expiry time.
-            Possible values include: 'NeverExpire', 'RelativeToCreation', 'RelativeToNow', 'Absolute'
-        :param datetime or int expires_on:
-            The time to set the file to expiry.
-            When expiry_options is RelativeTo*, expires_on should be an int in milliseconds
-        :keyword int timeout:
-            The timeout parameter is expressed in seconds.
-        :rtype: None
-        """
-        try:
-            expires_on = convert_datetime_to_rfc1123(expires_on)
-        except AttributeError:
-            expires_on = str(expires_on)
-        await self._datalake_client_for_blob_operation.path.set_expiry(expiry_options, expires_on=expires_on,
-                                                                       **kwargs)  # pylint: disable=protected-access
+        blob_properties = await self._get_path_properties(**kwargs)
+        return FileProperties._from_blob_properties(blob_properties)  # pylint: disable=protected-access
 
     async def upload_data(self, data,  # type: Union[AnyStr, Iterable[AnyStr], IO[AnyStr]]
                           length=None,  # type: Optional[int]
@@ -297,9 +255,6 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
             The match condition to use upon the etag.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
-        :keyword int chunk_size:
-            The maximum chunk size for uploading a file in chunks.
-            Defaults to 100*1024*1024, or 100MB.
         :return: response dict (Etag and last modified).
         """
         options = self._upload_options(
@@ -348,7 +303,7 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
             **kwargs)
         try:
             return await self._client.path.append_data(**options)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     async def flush_data(self, offset,  # type: int
@@ -415,7 +370,7 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
             retain_uncommitted_data=retain_uncommitted_data, **kwargs)
         try:
             return await self._client.path.flush_data(**options)
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
     async def download_file(self, offset=None, length=None, **kwargs):
@@ -472,8 +427,9 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
         downloader = await self._blob_client.download_blob(offset=offset, length=length, **kwargs)
         return StorageStreamDownloader(downloader)
 
-    async def rename_file(self, new_name, **kwargs):
-        # type: (str, **Any) -> DataLakeFileClient
+    async def rename_file(self, new_name,  # type: str
+                          **kwargs):
+        # type: (**Any) -> DataLakeFileClient
         """
         Rename the source file.
 
@@ -540,26 +496,14 @@ class DataLakeFileClient(PathClient, DataLakeFileClientBase):
         """
         new_name = new_name.strip('/')
         new_file_system = new_name.split('/')[0]
-        new_path_and_token = new_name[len(new_file_system):].strip('/').split('?')
-        new_path = new_path_and_token[0]
-        try:
-            new_file_sas = new_path_and_token[1] or self._query_str.strip('?')
-        except IndexError:
-            if not self._raw_credential and new_file_system != self.file_system_name:
-                raise ValueError("please provide the sas token for the new file")
-            if not self._raw_credential and new_file_system == self.file_system_name:
-                new_file_sas = self._query_str.strip('?')
+        path = new_name[len(new_file_system):]
 
-        new_file_client = DataLakeFileClient(
-            "{}://{}".format(self.scheme, self.primary_hostname), new_file_system, file_path=new_path,
-            credential=self._raw_credential or new_file_sas,
+        new_directory_client = DataLakeFileClient(
+            self.url, new_file_system, file_path=path, credential=self._raw_credential,
             _hosts=self._hosts, _configuration=self._config, _pipeline=self._pipeline,
             _location_mode=self._location_mode, require_encryption=self.require_encryption,
             key_encryption_key=self.key_encryption_key,
             key_resolver_function=self.key_resolver_function)
-        await new_file_client._rename_path(  # pylint: disable=protected-access
-            '/{}/{}{}'.format(quote(unquote(self.file_system_name)),
-                              quote(unquote(self.path_name)),
-                              self._query_str),
-            **kwargs)
-        return new_file_client
+        await new_directory_client._rename_path('/' + self.file_system_name + '/' + self.path_name, # pylint: disable=protected-access
+                                                **kwargs)
+        return new_directory_client
