@@ -26,7 +26,6 @@ except ImportError:
 import six
 
 from azure.core.configuration import Configuration
-from azure.core.credentials import AzureSasCredential
 from azure.core.exceptions import HttpResponseError
 from azure.core.pipeline import Pipeline
 from azure.core.pipeline.transport import RequestsTransport, HttpTransport
@@ -37,8 +36,6 @@ from azure.core.pipeline.policies import (
     ProxyPolicy,
     DistributedTracingPolicy,
     HttpLoggingPolicy,
-    UserAgentPolicy,
-    AzureSasCredentialPolicy
 )
 
 from .constants import STORAGE_OAUTH_SCOPE, SERVICE_HOST_BASE, CONNECTION_TIMEOUT, READ_TIMEOUT
@@ -47,6 +44,7 @@ from .authentication import SharedKeyCredentialPolicy
 from .shared_access_signature import QueryStringConstants
 from .policies import (
     StorageHeadersPolicy,
+    StorageUserAgentPolicy,
     StorageContentValidation,
     StorageRequestHook,
     StorageResponseHook,
@@ -55,7 +53,7 @@ from .policies import (
     QueueMessagePolicy,
     ExponentialRetry,
 )
-from .._version import VERSION
+from .._generated.models import StorageErrorException
 from .response_handlers import process_storage_error, PartialBatchErrorException
 
 
@@ -64,7 +62,6 @@ _SERVICE_PARAMS = {
     "blob": {"primary": "BlobEndpoint", "secondary": "BlobSecondaryEndpoint"},
     "queue": {"primary": "QueueEndpoint", "secondary": "QueueSecondaryEndpoint"},
     "file": {"primary": "FileEndpoint", "secondary": "FileSecondaryEndpoint"},
-    "dfs": {"primary": "BlobEndpoint", "secondary": "BlobEndpoint"},
 }
 
 
@@ -81,21 +78,16 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
         self._hosts = kwargs.get("_hosts")
         self.scheme = parsed_url.scheme
 
-        if service not in ["blob", "queue", "file-share", "dfs"]:
+        if service not in ["blob", "queue", "file-share"]:
             raise ValueError("Invalid service: {}".format(service))
         service_name = service.split('-')[0]
         account = parsed_url.netloc.split(".{}.core.".format(service_name))
-
         self.account_name = account[0] if len(account) > 1 else None
-        if not self.account_name and parsed_url.netloc.startswith("localhost") \
-                or parsed_url.netloc.startswith("127.0.0.1"):
-            self.account_name = parsed_url.path.strip("/")
+        secondary_hostname = None
 
-        self.credential = _format_shared_key_credential(self.account_name, credential)
+        self.credential = format_shared_key_credential(account, credential)
         if self.scheme.lower() != "https" and hasattr(self.credential, "get_token"):
             raise ValueError("Token credential is only supported with HTTPS.")
-
-        secondary_hostname = None
         if hasattr(self.credential, "account_name"):
             self.account_name = self.credential.account_name
             secondary_hostname = "{}-secondary.{}.{}".format(
@@ -196,23 +188,12 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
         else:
             raise ValueError("No host URL for location mode: {}".format(value))
 
-    @property
-    def api_version(self):
-        """The version of the Storage API used for requests.
-
-        :type: str
-        """
-        return self._client._config.version  # pylint: disable=protected-access
-
     def _format_query_string(self, sas_token, credential, snapshot=None, share_snapshot=None):
         query_str = "?"
         if snapshot:
             query_str += "snapshot={}&".format(self.snapshot)
         if share_snapshot:
             query_str += "sharesnapshot={}&".format(self.snapshot)
-        if sas_token and isinstance(credential, AzureSasCredential):
-            raise ValueError(
-                "You cannot use AzureSasCredential when the resource URI also contains a Shared Access Signature.")
         if sas_token and not credential:
             query_str += sas_token
         elif is_credential_sastoken(credential):
@@ -227,8 +208,6 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
             self._credential_policy = BearerTokenCredentialPolicy(credential, STORAGE_OAUTH_SCOPE)
         elif isinstance(credential, SharedKeyCredentialPolicy):
             self._credential_policy = credential
-        elif isinstance(credential, AzureSasCredential):
-            self._credential_policy = AzureSasCredentialPolicy(credential)
         elif credential is not None:
             raise TypeError("Unsupported credential: {}".format(credential))
 
@@ -242,23 +221,21 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
             config.transport = RequestsTransport(**kwargs)
         policies = [
             QueueMessagePolicy(),
+            config.headers_policy,
             config.proxy_policy,
             config.user_agent_policy,
             StorageContentValidation(),
-            ContentDecodePolicy(response_encoding="utf-8"),
+            StorageRequestHook(**kwargs),
+            self._credential_policy,
+            ContentDecodePolicy(),
             RedirectPolicy(**kwargs),
             StorageHosts(hosts=self._hosts, **kwargs),
             config.retry_policy,
-            config.headers_policy,
-            StorageRequestHook(**kwargs),
-            self._credential_policy,
             config.logging_policy,
             StorageResponseHook(**kwargs),
             DistributedTracingPolicy(**kwargs),
             HttpLoggingPolicy(**kwargs)
         ]
-        if kwargs.get("_additional_pipeline_policies"):
-            policies = policies + kwargs.get("_additional_pipeline_policies")
         return config, Pipeline(config.transport, policies=policies)
 
     def _batch_send(
@@ -272,7 +249,7 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
         request = self._client._client.post(  # pylint: disable=protected-access
             url='https://{}/?comp=batch'.format(self.primary_hostname),
             headers={
-                'x-ms-version': self.api_version
+                'x-ms-version': self._client._config.version  # pylint: disable=protected-access
             }
         )
 
@@ -281,8 +258,7 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
             policies=[
                 StorageHeadersPolicy(),
                 self._credential_policy
-            ],
-            enforce_https=False
+            ]
         )
 
         pipeline_response = self._pipeline.run(
@@ -304,7 +280,7 @@ class StorageAccountHostsMixin(object):  # pylint: disable=too-many-instance-att
                     raise error
                 return iter(parts)
             return parts
-        except HttpResponseError as error:
+        except StorageErrorException as error:
             process_storage_error(error)
 
 class TransportWrapper(HttpTransport):
@@ -331,11 +307,11 @@ class TransportWrapper(HttpTransport):
         pass
 
 
-def _format_shared_key_credential(account_name, credential):
+def format_shared_key_credential(account, credential):
     if isinstance(credential, six.string_types):
-        if not account_name:
+        if len(account) < 2:
             raise ValueError("Unable to determine account name for shared key credential.")
-        credential = {"account_name": account_name, "account_key": credential}
+        credential = {"account_name": account[0], "account_key": credential}
     if isinstance(credential, dict):
         if "account_name" not in credential:
             raise ValueError("Shared key credential missing 'account_name")
@@ -348,7 +324,7 @@ def _format_shared_key_credential(account_name, credential):
 def parse_connection_str(conn_str, credential, service):
     conn_str = conn_str.rstrip(";")
     conn_settings = [s.split("=", 1) for s in conn_str.split(";")]
-    if any(len(tup) != 2 for tup in conn_settings):
+    if  any(len(tup) != 2 for tup in conn_settings):
         raise ValueError("Connection string is either blank or malformed.")
     conn_settings = dict(conn_settings)
     endpoints = _SERVICE_PARAMS[service]
@@ -393,8 +369,7 @@ def create_configuration(**kwargs):
     # type: (**Any) -> Configuration
     config = Configuration(**kwargs)
     config.headers_policy = StorageHeadersPolicy(**kwargs)
-    config.user_agent_policy = UserAgentPolicy(
-        sdk_moniker="storage-{}/{}".format(kwargs.pop('storage_sdk'), VERSION), **kwargs)
+    config.user_agent_policy = StorageUserAgentPolicy(**kwargs)
     config.retry_policy = kwargs.get("retry_policy") or ExponentialRetry(**kwargs)
     config.logging_policy = StorageLoggingPolicy(**kwargs)
     config.proxy_policy = ProxyPolicy(**kwargs)
