@@ -6,14 +6,19 @@
 # pylint: disable=too-few-public-methods, too-many-instance-attributes
 # pylint: disable=super-init-not-called, too-many-lines
 
+from enum import Enum
+
 from azure.core.paging import PageIterator
+from azure.core.exceptions import HttpResponseError
 from ._parser import _parse_datetime_from_str
 from ._shared.response_handlers import return_context_and_deserialized, process_storage_error
 from ._shared.models import DictMixin, get_enum_value
-from ._generated.models import StorageErrorException
 from ._generated.models import Metrics as GeneratedMetrics
 from ._generated.models import RetentionPolicy as GeneratedRetentionPolicy
 from ._generated.models import CorsRule as GeneratedCorsRule
+from ._generated.models import ShareProtocolSettings as GeneratedShareProtocolSettings
+from ._generated.models import ShareSmbSettings as GeneratedShareSmbSettings
+from ._generated.models import SmbMultichannel as GeneratedSmbMultichannel
 from ._generated.models import AccessPolicy as GenAccessPolicy
 from ._generated.models import DirectoryItem
 
@@ -134,6 +139,46 @@ class CorsRule(GeneratedCorsRule):
         )
 
 
+class ShareSmbSettings(GeneratedShareSmbSettings):
+    """ Settings for the SMB protocol.
+
+    :keyword SmbMultichannel multichannel: Sets the multichannel settings.
+    """
+    def __init__(self, **kwargs):
+        self.multichannel = kwargs.get('multichannel')
+        if self.multichannel is None:
+            raise ValueError("The value 'multichannel' must be specified.")
+
+
+class SmbMultichannel(GeneratedSmbMultichannel):
+    """ Settings for Multichannel.
+
+    :keyword bool enabled: If SMB Multichannel is enabled.
+    """
+    def __init__(self, **kwargs):
+        self.enabled = kwargs.get('enabled')
+        if self.enabled is None:
+            raise ValueError("The value 'enabled' must be specified.")
+
+
+class ShareProtocolSettings(GeneratedShareProtocolSettings):
+    """Protocol Settings class used by the set and get service properties methods in the share service.
+
+    Contains protocol properties of the share service such as the SMB setting of the share service.
+
+    :keyword SmbSettings smb: Sets SMB settings.
+    """
+    def __init__(self, **kwargs):
+        self.smb = kwargs.get('smb')
+        if self.smb is None:
+            raise ValueError("The value 'smb' must be specified.")
+
+    @classmethod
+    def _from_generated(cls, generated):
+        return cls(
+            smb=generated.smb)
+
+
 class AccessPolicy(GenAccessPolicy):
     """Access Policy class used by the set and get acl methods in each service.
 
@@ -185,14 +230,14 @@ class AccessPolicy(GenAccessPolicy):
 
 
 class LeaseProperties(DictMixin):
-    """File Lease Properties.
+    """File or Share Lease Properties.
 
     :ivar str status:
-        The lease status of the file. Possible values: locked|unlocked
+        The lease status of the file or share. Possible values: locked|unlocked
     :ivar str state:
-        Lease state of the file. Possible values: available|leased|expired|breaking|broken
+        Lease state of the file or share. Possible values: available|leased|expired|breaking|broken
     :ivar str duration:
-        When a file is leased, specifies whether the lease is of infinite or fixed duration.
+        When a file or share is leased, specifies whether the lease is of infinite or fixed duration.
     """
 
     def __init__(self, **kwargs):
@@ -229,7 +274,7 @@ class ContentSettings(DictMixin):
     :param str cache_control:
         If the cache_control has previously been set for
         the file, that value is stored.
-    :param str content_md5:
+    :param bytearray content_md5:
         If the content_md5 has been set for the file, this response
         header is stored so that the client can check for message content
         integrity.
@@ -271,6 +316,8 @@ class ShareProperties(DictMixin):
         conditionally.
     :ivar int quota:
         The allocated quota.
+    :ivar str access_tier:
+        The share's access tier.
     :ivar dict metadata: A dict with name_value pairs to associate with the
         share as metadata.
     :ivar str snapshot:
@@ -287,6 +334,10 @@ class ShareProperties(DictMixin):
     :ivar int remaining_retention_days:
         To indicate how many remaining days the deleted share will be kept.
         This is a service returned value, and the value will be set when list shared including deleted ones.
+    :ivar ~azure.storage.fileshare.models.ShareRootSquash or str root_squash:
+        Possible values include: 'NoRootSquash', 'RootSquash', 'AllSquash'.
+    :ivar list(str) protocols:
+        Indicates the protocols enabled on the share. The protocol can be either SMB or NFS.
     """
 
     def __init__(self, **kwargs):
@@ -294,6 +345,7 @@ class ShareProperties(DictMixin):
         self.last_modified = kwargs.get('Last-Modified')
         self.etag = kwargs.get('ETag')
         self.quota = kwargs.get('x-ms-share-quota')
+        self.access_tier = kwargs.get('x-ms-access-tier')
         self.next_allowed_quota_downgrade_time = kwargs.get('x-ms-share-next-allowed-quota-downgrade-time')
         self.metadata = kwargs.get('metadata')
         self.snapshot = None
@@ -304,7 +356,10 @@ class ShareProperties(DictMixin):
         self.provisioned_egress_mbps = kwargs.get('x-ms-share-provisioned-egress-mbps')
         self.provisioned_ingress_mbps = kwargs.get('x-ms-share-provisioned-ingress-mbps')
         self.provisioned_iops = kwargs.get('x-ms-share-provisioned-iops')
-
+        self.lease = LeaseProperties(**kwargs)
+        self.protocols = [protocol.strip() for protocol in kwargs.get('x-ms-enabled-protocols', None).split(',')]\
+            if kwargs.get('x-ms-enabled-protocols', None) else None
+        self.root_squash = kwargs.get('x-ms-root-squash', None)
     @classmethod
     def _from_generated(cls, generated):
         props = cls()
@@ -312,6 +367,7 @@ class ShareProperties(DictMixin):
         props.last_modified = generated.properties.last_modified
         props.etag = generated.properties.etag
         props.quota = generated.properties.quota
+        props.access_tier = generated.properties.access_tier
         props.next_allowed_quota_downgrade_time = generated.properties.next_allowed_quota_downgrade_time
         props.metadata = generated.metadata
         props.snapshot = generated.snapshot
@@ -319,9 +375,14 @@ class ShareProperties(DictMixin):
         props.deleted_time = generated.properties.deleted_time
         props.version = generated.version
         props.remaining_retention_days = generated.properties.remaining_retention_days
-        props.provisioned_egress_mbps = generated.properties.provisioned_egress_mbps
-        props.provisioned_ingress_mbps = generated.properties.provisioned_ingress_mbps
+        props.provisioned_egress_mbps = generated.properties.provisioned_egress_m_bps
+        props.provisioned_ingress_mbps = generated.properties.provisioned_ingress_m_bps
         props.provisioned_iops = generated.properties.provisioned_iops
+        props.lease = LeaseProperties._from_generated(generated)  # pylint: disable=protected-access
+        props.protocols = [protocol.strip() for protocol in generated.properties.enabled_protocols.split(',')]\
+            if generated.properties.enabled_protocols else None
+        props.root_squash = generated.properties.root_squash
+
         return props
 
 
@@ -367,7 +428,7 @@ class SharePropertiesPaged(PageIterator):
                 prefix=self.prefix,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -459,7 +520,7 @@ class HandlesPaged(PageIterator):
                 maxresults=self.results_per_page,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -573,7 +634,7 @@ class DirectoryPropertiesPaged(PageIterator):
                 maxresults=self.results_per_page,
                 cls=return_context_and_deserialized,
                 use_location=self.location_mode)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     def _extract_data_cb(self, get_next_return):
@@ -657,6 +718,12 @@ class FileProperties(DictMixin):
         return props
 
 
+class ShareProtocols(str, Enum):
+    """Enabled protocols on the share"""
+    SMB = "SMB"
+    NFS = "NFS"
+
+
 class CopyProperties(DictMixin):
     """File Copy Properties.
 
@@ -773,7 +840,7 @@ class FileSasPermissions(object):
         p_delete = 'd' in permission
 
         parsed = cls(p_read, p_create, p_write, p_delete)
-        parsed._str = permission # pylint: disable = protected-access
+
         return parsed
 
 
@@ -830,7 +897,7 @@ class ShareSasPermissions(object):
         p_list = 'l' in permission
 
         parsed = cls(p_read, p_write, p_delete, p_list)
-        parsed._str = permission # pylint: disable = protected-access
+
         return parsed
 
 class NTFSAttributes(object):
@@ -922,4 +989,5 @@ def service_properties_deserialize(generated):
         'hour_metrics': Metrics._from_generated(generated.hour_metrics),  # pylint: disable=protected-access
         'minute_metrics': Metrics._from_generated(generated.minute_metrics),  # pylint: disable=protected-access
         'cors': [CorsRule._from_generated(cors) for cors in generated.cors],  # pylint: disable=protected-access
+        'protocol': ShareProtocolSettings._from_generated(generated.protocol), # pylint: disable=protected-access
     }
