@@ -3,14 +3,15 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-
+# pylint: disable=too-many-lines, invalid-overridden-method, too-many-public-methods
 import functools
 import time
 from io import BytesIO
-from typing import Optional, Union, IO, List, Dict, Any, Iterable, TYPE_CHECKING  # pylint: disable=unused-import
+from typing import Optional, Union, IO, List, Tuple, Dict, Any, Iterable, TYPE_CHECKING  # pylint: disable=unused-import
 
 import six
 from azure.core.async_paging import AsyncItemPaged
+from azure.core.exceptions import HttpResponseError
 
 from azure.core.tracing.decorator import distributed_trace
 from azure.core.tracing.decorator_async import distributed_trace_async
@@ -18,16 +19,17 @@ from .._parser import _datetime_to_str, _get_file_permission
 from .._shared.parser import _str
 
 from .._generated.aio import AzureFileStorage
-from .._generated.version import VERSION
-from .._generated.models import StorageErrorException, FileHTTPHeaders
+from .._generated.models import FileHTTPHeaders
 from .._shared.policies_async import ExponentialRetry
 from .._shared.uploads_async import upload_data_chunks, FileChunkUploader, IterStreamer
 from .._shared.base_client_async import AsyncStorageAccountHostsMixin
 from .._shared.request_handlers import add_metadata_headers, get_length
 from .._shared.response_handlers import return_response_headers, process_storage_error
-from .._deserialize import deserialize_file_properties, deserialize_file_stream
+from .._deserialize import deserialize_file_properties, deserialize_file_stream, get_file_ranges_result
+from .._serialize import get_access_conditions, get_smb_properties, get_api_version
 from .._file_client import ShareFileClient as ShareFileClientBase
 from ._models import HandlesPaged
+from ._lease_async import ShareLeaseClient
 from ._download_async import StorageStreamDownloader
 
 if TYPE_CHECKING:
@@ -81,7 +83,7 @@ async def _upload_file_helper(
             **kwargs
         )
         return sorted(responses, key=lambda r: r.get('last_modified'))[-1]
-    except StorageErrorException as error:
+    except HttpResponseError as error:
         process_storage_error(error)
 
 
@@ -102,8 +104,15 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         or the response returned from :func:`ShareClient.create_snapshot`.
     :param credential:
         The credential with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string or an account
+        account URL already has a SAS token. The value can be a SAS token string,
+        an instance of a AzureSasCredential from azure.core.credentials or an account
         shared access key.
+    :keyword str api_version:
+        The Storage API version to use for requests. Default value is '2019-07-07'.
+        Setting to an older version may result in reduced feature compatibility.
+
+        .. versionadded:: 12.1.0
+
     :keyword str secondary_hostname:
         The hostname of the secondary endpoint.
     :keyword loop:
@@ -127,9 +136,42 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             account_url, share_name=share_name, file_path=file_path, snapshot=snapshot,
             credential=credential, loop=loop, **kwargs
         )
-        self._client = AzureFileStorage(version=VERSION, url=self.url, pipeline=self._pipeline, loop=loop)
+        self._client = AzureFileStorage(url=self.url, pipeline=self._pipeline, loop=loop)
+        default_api_version = self._client._config.version  # pylint: disable=protected-access
+        self._client._config.version = get_api_version(kwargs, default_api_version) # pylint: disable=protected-access
         self._loop = loop
 
+    @distributed_trace_async
+    async def acquire_lease(self, lease_id=None, **kwargs):
+        # type: (Optional[str], **Any) -> ShareLeaseClient
+        """Requests a new lease.
+
+        If the file does not have an active lease, the File
+        Service creates a lease on the blob and returns a new lease.
+
+        :param str lease_id:
+            Proposed lease ID, in a GUID string format. The File Service
+            returns 400 (Invalid request) if the proposed lease ID is not
+            in the correct format.
+        :keyword int timeout:
+            The timeout parameter is expressed in seconds.
+        :returns: A ShareLeaseClient object.
+        :rtype: ~azure.storage.fileshare.aio.ShareLeaseClient
+
+        .. admonition:: Example:
+
+            .. literalinclude:: ../samples/blob_samples_common.py
+                :start-after: [START acquire_lease_on_blob]
+                :end-before: [END acquire_lease_on_blob]
+                :language: python
+                :dedent: 8
+                :caption: Acquiring a lease on a blob.
+        """
+        kwargs['lease_duration'] = -1
+        lease = ShareLeaseClient(self, lease_id=lease_id)  # type: ignore
+        await lease.acquire(**kwargs)
+        return lease
+
     @distributed_trace_async
     async def create_file(  # type: ignore
         self,
@@ -176,6 +218,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             language, disposition, md5, and cache control.
         :keyword dict(str,str) metadata:
             Name-value pairs associated with the file as metadata.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
@@ -190,6 +239,7 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 :dedent: 16
                 :caption: Create a file.
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         content_settings = kwargs.pop('content_settings', None)
         metadata = kwargs.pop('metadata', None)
         timeout = kwargs.pop('timeout', None)
@@ -219,12 +269,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 file_permission=file_permission,
                 file_permission_key=permission_key,
                 file_http_headers=file_http_headers,
+                lease_access_conditions=access_conditions,
                 headers=headers,
                 timeout=timeout,
                 cls=return_response_headers,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -284,6 +335,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             Maximum number of parallel connections to use.
         :keyword str encoding:
             Defaults to UTF-8.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
@@ -341,12 +399,8 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         )
 
     @distributed_trace_async
-    async def start_copy_from_url(
-        self,
-        source_url,  # type: str
-        **kwargs  # type: Any
-    ):
-        # type: (...) -> Any
+    async def start_copy_from_url(self, source_url, **kwargs):
+        # type: (str, Any) -> Any
         """Initiates the copying of data from a source URL into the file
         referenced by the client.
 
@@ -355,8 +409,80 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
 
         :param str source_url:
             Specifies the URL of the source file.
-        :keyword dict(str,str) metadata:
+        :keyword str file_permission:
+            If specified the permission (security descriptor) shall be set for the directory/file.
+            This value can be set to "source" to copy the security descriptor from the source file.
+            Otherwise if set, this value will be used to override the source value. If not set, permission value
+            is inherited from the parent directory of the target file. This setting can be
+            used if Permission size is <= 8KB, otherwise permission_key shall be used.
+            If SDDL is specified as input, it must have owner, group and dacl.
+            Note: Only one of the file_permission or permission_key should be specified.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :keyword str permission_key:
+            Key of the permission to be set for the directory/file.
+            This value can be set to "source" to copy the security descriptor from the source file.
+            Otherwise if set, this value will be used to override the source value. If not set, permission value
+            is inherited from the parent directory of the target file.
+            Note: Only one of the file_permission or permission_key should be specified.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :keyword file_attributes:
+            This value can be set to "source" to copy file attributes from the source file to the target file,
+            or to clear all attributes, it can be set to "None". Otherwise it can be set to a list of attributes
+            to set on the target file. If this is not set, the default value is "Archive".
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :paramtype file_attributes: str or :class:`~azure.storage.fileshare.NTFSAttributes`
+        :keyword file_creation_time:
+            This value can be set to "source" to copy the creation time from the source file to the target file,
+            or a datetime to set as creation time on the target file. This could also be a string in ISO 8601 format.
+            If this is not set, creation time will be set to the date time value of the creation
+            (or when it was overwritten) of the target file by copy engine.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :paramtype file_creation_time: str or ~datetime.datetime
+        :keyword file_last_write_time:
+            This value can be set to "source" to copy the last write time from the source file to the target file, or
+            a datetime to set as the last write time on the target file. This could also be a string in ISO 8601 format.
+            If this is not set, value will be the last write time to the file by the copy engine.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :paramtype file_last_write_time: str or ~datetime.datetime
+        :keyword bool ignore_read_only:
+            Specifies the option to overwrite the target file if it already exists and has read-only attribute set.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :keyword bool set_archive_attribute:
+            Specifies the option to set the archive attribute on the target file.
+            True means the archive attribute will be set on the target file despite attribute
+            overrides or the source file state.
+
+            .. versionadded:: 12.1.0
+                This parameter was introduced in API version '2019-07-07'.
+
+        :keyword metadata:
             Name-value pairs associated with the file as metadata.
+        :type metadata: dict(str, str)
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :rtype: dict(str, Any)
@@ -371,15 +497,22 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 :caption: Copy a file from a URL
         """
         metadata = kwargs.pop('metadata', None)
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         headers = kwargs.pop("headers", {})
         headers.update(add_metadata_headers(metadata))
-
+        kwargs.update(get_smb_properties(kwargs))
         try:
             return await self._client.file.start_copy(
-                source_url, timeout=timeout, metadata=metadata, headers=headers, cls=return_response_headers, **kwargs
+                source_url,
+                metadata=metadata,
+                lease_access_conditions=access_conditions,
+                headers=headers,
+                cls=return_response_headers,
+                timeout=timeout,
+                **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -394,10 +527,18 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             The copy operation to abort. This can be either an ID, or an
             instance of FileProperties.
         :type copy_id: str or ~azure.storage.fileshare.FileProperties
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :rtype: None
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         try:
             copy_id = copy_id.copy.id
@@ -407,8 +548,10 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             except TypeError:
                 pass
         try:
-            await self._client.file.abort_copy(copy_id=copy_id, timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+            await self._client.file.abort_copy(copy_id=copy_id,
+                                               lease_access_conditions=access_conditions,
+                                               timeout=timeout, **kwargs)
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -418,8 +561,10 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         length=None,  # type: Optional[int]
         **kwargs
     ):
-        # type: (...) -> Iterable[bytes]
-        """Downloads a file to a stream with automatic chunking.
+        # type: (Optional[int], Optional[int], Any) -> StorageStreamDownloader
+        """Downloads a file to the StorageStreamDownloader. The readall() method must
+        be used to read all the content or readinto() must be used to download the file into
+        a stream. Using chunks() returns an async iterator which allows the user to iterate over the content in chunks.
 
         :param int offset:
             Start of byte range to use for downloading a section of the file.
@@ -438,9 +583,17 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             file. Also note that if enabled, the memory-efficient upload algorithm
             will not be used, because computing the MD5 hash requires buffering
             entire blocks, and doing so defeats the purpose of the memory-efficient algorithm.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
-        :returns: A iterable data generator (stream)
+        :returns: A streaming object (StorageStreamDownloader)
+        :rtype: ~azure.storage.fileshare.aio.StorageStreamDownloader
 
         .. admonition:: Example:
 
@@ -459,6 +612,9 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         range_end = None
         if length is not None:
             range_end = offset + length - 1  # Service actually uses an end-range inclusive index
+
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
+
         downloader = StorageStreamDownloader(
             client=self._client.file,
             config=self._config,
@@ -468,6 +624,7 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             name=self.file_name,
             path='/'.join(self.file_path),
             share=self.share_name,
+            lease_access_conditions=access_conditions,
             cls=deserialize_file_stream,
             **kwargs
         )
@@ -480,6 +637,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         """Marks the specified file for deletion. The file is
         later deleted during garbage collection.
 
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :rtype: None
@@ -493,10 +657,11 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 :dedent: 16
                 :caption: Delete a file.
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         try:
-            await self._client.file.delete(timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+            await self._client.file.delete(lease_access_conditions=access_conditions, timeout=timeout, **kwargs)
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -505,17 +670,29 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         """Returns all user-defined metadata, standard HTTP properties, and
         system properties for the file.
 
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: FileProperties
         :rtype: ~azure.storage.fileshare.FileProperties
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         try:
             file_props = await self._client.file.get_properties(
-                sharesnapshot=self.snapshot, timeout=timeout, cls=deserialize_file_properties, **kwargs
+                sharesnapshot=self.snapshot,
+                lease_access_conditions=access_conditions,
+                timeout=timeout,
+                cls=deserialize_file_properties,
+                **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         file_props.name = self.file_name
         file_props.share = self.share_name
@@ -560,11 +737,19 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             directory/file. Note: Only one of the x-ms-file-permission or
             x-ms-file-permission-key should be specified.
         :type permission_key: str
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
         :rtype: dict(str, Any)
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         file_content_length = kwargs.pop("size", None)
         file_http_headers = FileHTTPHeaders(
@@ -585,11 +770,12 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 file_last_write_time=_datetime_to_str(file_last_write_time),
                 file_permission=file_permission,
                 file_permission_key=permission_key,
+                lease_access_conditions=access_conditions,
                 timeout=timeout,
                 cls=return_response_headers,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -605,19 +791,28 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         :param metadata:
             Name-value pairs associated with the file as metadata.
         :type metadata: dict(str, str)
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
         :rtype: dict(str, Any)
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         headers = kwargs.pop("headers", {})
         headers.update(add_metadata_headers(metadata))  # type: ignore
         try:
             return await self._client.file.set_metadata(  # type: ignore
-                timeout=timeout, cls=return_response_headers, headers=headers, metadata=metadata, **kwargs
+                metadata=metadata, lease_access_conditions=access_conditions,
+                timeout=timeout, cls=return_response_headers, headers=headers, **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -646,6 +841,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             bitflips on the wire if using http instead of https as https (the default)
             will already validate. Note that this MD5 hash is not stored with the
             file.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :keyword str encoding:
@@ -662,6 +864,7 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             data = data.encode(encoding)
         end_range = offset + length - 1  # Reformat to an inclusive range index
         content_range = 'bytes={0}-{1}'.format(offset, end_range)
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         try:
             return await self._client.file.upload_range(  # type: ignore
                 range=content_range,
@@ -669,10 +872,11 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 optionalbody=data,
                 timeout=timeout,
                 validate_content=validate_content,
+                lease_access_conditions=access_conditions,
                 cls=return_response_headers,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -704,6 +908,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         :param int source_offset:
             This indicates the start of the range of bytes(inclusive) that has to be taken from the copy source.
             The service will read the same number of bytes as the destination range (length-offset).
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         """
@@ -716,46 +927,89 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         )
         try:
             return await self._client.file.upload_range_from_url(**options)  # type: ignore
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
     async def get_ranges(  # type: ignore
-        self,
-        offset=None,  # type: Optional[int]
-        length=None,  # type: Optional[int]
-        **kwargs
-    ):
+            self, offset=None,  # type: Optional[int]
+            length=None,  # type: Optional[int]
+            **kwargs  # type: Any
+        ):
         # type: (...) -> List[Dict[str, int]]
-        """Returns the list of valid ranges of a file.
+        """Returns the list of valid page ranges for a file or snapshot
+        of a file.
 
         :param int offset:
             Specifies the start offset of bytes over which to get ranges.
         :param int length:
-            Number of bytes to use over which to get ranges.
+           Number of bytes to use over which to get ranges.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
-        :returns: A list of valid ranges.
+        :returns:
+            A list of valid ranges.
         :rtype: List[dict[str, int]]
         """
-        timeout = kwargs.pop('timeout', None)
-        if self.require_encryption or (self.key_encryption_key is not None):
-            raise ValueError("Unsupported method for encryption.")
-
-        content_range = None
-        if offset is not None:
-            if length is not None:
-                end_range = offset + length - 1  # Reformat to an inclusive range index
-                content_range = "bytes={0}-{1}".format(offset, end_range)
-            else:
-                content_range = "bytes={0}-".format(offset)
+        options = self._get_ranges_options(
+            offset=offset,
+            length=length,
+            **kwargs)
         try:
-            ranges = await self._client.file.get_range_list(
-                sharesnapshot=self.snapshot, timeout=timeout, range=content_range, **kwargs
-            )
-        except StorageErrorException as error:
+            ranges = await self._client.file.get_range_list(**options)
+        except HttpResponseError as error:
             process_storage_error(error)
-        return [{"start": b.start, "end": b.end} for b in ranges]
+        return [{'start': file_range.start, 'end': file_range.end} for file_range in ranges.ranges]
+
+    @distributed_trace_async
+    async def get_ranges_diff(  # type: ignore
+            self,
+            previous_sharesnapshot,  # type: Union[str, Dict[str, Any]]
+            offset=None,  # type: Optional[int]
+            length=None,  # type: Optional[int]
+            **kwargs  # type: Any
+            ):
+        # type: (...) -> Tuple[List[Dict[str, int]], List[Dict[str, int]]]
+        """Returns the list of valid page ranges for a file or snapshot
+        of a file.
+
+        .. versionadded:: 12.6.0
+
+        :param int offset:
+            Specifies the start offset of bytes over which to get ranges.
+        :param int length:
+           Number of bytes to use over which to get ranges.
+        :param str previous_sharesnapshot:
+            The snapshot diff parameter that contains an opaque DateTime value that
+            specifies a previous file snapshot to be compared
+            against a more recent snapshot or the current file.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+        :paramtype lease: ~azure.storage.fileshare.ShareLeaseClient or str
+        :keyword int timeout:
+            The timeout parameter is expressed in seconds.
+        :returns:
+            A tuple of two lists of file ranges as dictionaries with 'start' and 'end' keys.
+            The first element are filled file ranges, the 2nd element is cleared file ranges.
+        :rtype: tuple(list(dict(str, str), list(dict(str, str))
+        """
+        options = self._get_ranges_options(
+            offset=offset,
+            length=length,
+            previous_sharesnapshot=previous_sharesnapshot,
+            **kwargs)
+        try:
+            ranges = await self._client.file.get_range_list(**options)
+        except HttpResponseError as error:
+            process_storage_error(error)
+        return get_file_ranges_result(ranges)
 
     @distributed_trace_async
     async def clear_range(  # type: ignore
@@ -774,11 +1028,19 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         :param int length:
             Number of bytes to use for clearing a section of the file.
             The range can be up to 4 MB in size.
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
         :rtype: Dict[str, Any]
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         if self.require_encryption or (self.key_encryption_key is not None):
             raise ValueError("Unsupported method for encryption.")
@@ -794,11 +1056,13 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 timeout=timeout,
                 cls=return_response_headers,
                 content_length=0,
+                optionalbody=None,
                 file_range_write="clear",
                 range=content_range,
+                lease_access_conditions=access_conditions,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -808,11 +1072,19 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
 
         :param int size:
             Size to resize file to (in bytes)
+        :keyword lease:
+            Required if the file has an active lease. Value can be a ShareLeaseClient object
+            or the lease ID as a string.
+
+            .. versionadded:: 12.1.0
+
+        :paramtype lease: ~azure.storage.fileshare.aio.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: File-updated property dict (Etag and last modified).
         :rtype: Dict[str, Any]
         """
+        access_conditions = get_access_conditions(kwargs.pop('lease', None))
         timeout = kwargs.pop('timeout', None)
         try:
             return await self._client.file.set_http_headers(  # type: ignore
@@ -821,11 +1093,12 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                 file_creation_time="preserve",
                 file_last_write_time="preserve",
                 file_permission="preserve",
+                lease_access_conditions=access_conditions,
                 cls=return_response_headers,
                 timeout=timeout,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -880,8 +1153,9 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
             )
             return {
                 'closed_handles_count': response.get('number_of_handles_closed', 0),
+                'failed_handles_count': response.get('number_of_handles_failed', 0)
             }
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace_async
@@ -904,6 +1178,7 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
         try_close = True
         continuation_token = None
         total_closed = 0
+        total_failed = 0
         while try_close:
             try:
                 response = await self._client.file.force_close_handles(
@@ -914,13 +1189,15 @@ class ShareFileClient(AsyncStorageAccountHostsMixin, ShareFileClientBase):
                     cls=return_response_headers,
                     **kwargs
                 )
-            except StorageErrorException as error:
+            except HttpResponseError as error:
                 process_storage_error(error)
             continuation_token = response.get('marker')
             try_close = bool(continuation_token)
             total_closed += response.get('number_of_handles_closed', 0)
+            total_failed += response.get('number_of_handles_failed', 0)
             if timeout:
                 timeout = max(0, timeout - (time.time() - start_time))
         return {
             'closed_handles_count': total_closed,
+            'failed_handles_count': total_failed
         }
