@@ -12,6 +12,7 @@ from typing import ( # pylint: disable=unused-import
     TYPE_CHECKING
 )
 
+
 try:
     from urllib.parse import urlparse, quote, unquote
 except ImportError:
@@ -19,12 +20,12 @@ except ImportError:
     from urllib2 import quote, unquote # type: ignore
 
 import six
+from azure.core.exceptions import HttpResponseError
 from azure.core.paging import ItemPaged  # pylint: disable=ungrouped-imports
 from azure.core.tracing.decorator import distributed_trace
 
 from ._generated import AzureFileStorage
-from ._generated.version import VERSION
-from ._generated.models import StorageErrorException, FileHTTPHeaders
+from ._generated.models import FileHTTPHeaders
 from ._shared.uploads import IterStreamer, FileChunkUploader, upload_data_chunks
 from ._shared.base_client import StorageAccountHostsMixin, parse_connection_str, parse_query
 from ._shared.request_handlers import add_metadata_headers, get_length
@@ -89,13 +90,17 @@ def _upload_file_helper(
             **kwargs
         )
         return sorted(responses, key=lambda r: r.get('last_modified'))[-1]
-    except StorageErrorException as error:
+    except HttpResponseError as error:
         process_storage_error(error)
 
 
 class ShareFileClient(StorageAccountHostsMixin):
     """A client to interact with a specific file, although that file may not yet exist.
 
+    For more optional configuration, please click
+    `here <https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/storage/azure-storage-file-share
+    #optional-configuration>`_.
+
     :param str account_url:
         The URI to the storage account. In order to create a client given the full URI to the
         file, use the :func:`from_file_url` classmethod.
@@ -110,7 +115,8 @@ class ShareFileClient(StorageAccountHostsMixin):
         or the response returned from :func:`ShareClient.create_snapshot`.
     :param credential:
         The credential with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string or an account
+        account URL already has a SAS token. The value can be a SAS token string,
+        an instance of a AzureSasCredential from azure.core.credentials or an account
         shared access key.
     :keyword str api_version:
         The Storage API version to use for requests. Default value is '2019-07-07'.
@@ -165,8 +171,8 @@ class ShareFileClient(StorageAccountHostsMixin):
         self._query_str, credential = self._format_query_string(
             sas_token, credential, share_snapshot=self.snapshot)
         super(ShareFileClient, self).__init__(parsed_url, service='file-share', credential=credential, **kwargs)
-        self._client = AzureFileStorage(version=VERSION, url=self.url, pipeline=self._pipeline)
-        self._client._config.version = get_api_version(kwargs, VERSION)  # pylint: disable=protected-access
+        self._client = AzureFileStorage(url=self.url, pipeline=self._pipeline)
+        self._client._config.version = get_api_version(kwargs) # pylint: disable=protected-access
 
     @classmethod
     def from_file_url(
@@ -184,7 +190,8 @@ class ShareFileClient(StorageAccountHostsMixin):
             or the response returned from :func:`ShareClient.create_snapshot`.
         :param credential:
             The credential with which to authenticate. This is optional if the
-            account URL already has a SAS token. The value can be a SAS token string or an account
+            account URL already has a SAS token. The value can be a SAS token string,
+            an instance of a AzureSasCredential from azure.core.credentials or an account
             shared access key.
         :returns: A File client.
         :rtype: ~azure.storage.fileshare.ShareFileClient
@@ -244,7 +251,8 @@ class ShareFileClient(StorageAccountHostsMixin):
             or the response returned from :func:`ShareClient.create_snapshot`.
         :param credential:
             The credential with which to authenticate. This is optional if the
-            account URL already has a SAS token. The value can be a SAS token string or an account
+            account URL already has a SAS token. The value can be a SAS token string,
+            an instance of a AzureSasCredential from azure.core.credentials or an account
             shared access key.
         :returns: A File client.
         :rtype: ~azure.storage.fileshare.ShareFileClient
@@ -396,7 +404,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=return_response_headers,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -631,7 +639,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 cls=return_response_headers,
                 timeout=timeout,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     def abort_copy(self, copy_id, **kwargs):
@@ -669,7 +677,7 @@ class ShareFileClient(StorageAccountHostsMixin):
             self._client.file.abort_copy(copy_id=copy_id,
                                          lease_access_conditions=access_conditions,
                                          timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -678,8 +686,10 @@ class ShareFileClient(StorageAccountHostsMixin):
             length=None,  # type: Optional[int]
             **kwargs
         ):
-        # type: (...) -> Iterable[bytes]
-        """Downloads a file to a stream with automatic chunking.
+        # type: (Optional[int], Optional[int], Any) -> StorageStreamDownloader
+        """Downloads a file to the StorageStreamDownloader. The readall() method must
+        be used to read all the content or readinto() must be used to download the file into
+        a stream. Using chunks() returns an iterator which allows the user to iterate over the content in chunks.
 
         :param int offset:
             Start of byte range to use for downloading a section of the file.
@@ -707,7 +717,8 @@ class ShareFileClient(StorageAccountHostsMixin):
         :paramtype lease: ~azure.storage.fileshare.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
-        :returns: A iterable data generator (stream)
+        :returns: A streaming object (StorageStreamDownloader)
+        :rtype: ~azure.storage.fileshare.StorageStreamDownloader
 
         .. admonition:: Example:
 
@@ -772,7 +783,7 @@ class ShareFileClient(StorageAccountHostsMixin):
         timeout = kwargs.pop('timeout', None)
         try:
             self._client.file.delete(lease_access_conditions=access_conditions, timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -802,7 +813,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=deserialize_file_properties,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         file_props.name = self.file_name
         file_props.share = self.share_name
@@ -884,7 +895,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=return_response_headers,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -924,7 +935,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 metadata=metadata,
                 lease_access_conditions=access_conditions,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -987,7 +998,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 lease_access_conditions=access_conditions,
                 cls=return_response_headers,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @staticmethod
@@ -1010,11 +1021,12 @@ class ShareFileClient(StorageAccountHostsMixin):
         end_range = offset + length - 1
         destination_range = 'bytes={0}-{1}'.format(offset, end_range)
         source_range = 'bytes={0}-{1}'.format(source_offset, source_offset + length - 1)
-
+        source_authorization = kwargs.pop('source_authorization', None)
         source_mod_conditions = get_source_conditions(kwargs)
         access_conditions = get_access_conditions(kwargs.pop('lease', None))
 
         options = {
+            'copy_source_authorization': source_authorization,
             'copy_source': source_url,
             'content_length': 0,
             'source_range': source_range,
@@ -1081,6 +1093,9 @@ class ShareFileClient(StorageAccountHostsMixin):
         :paramtype lease: ~azure.storage.fileshare.ShareLeaseClient or str
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
+        :keyword str source_authorization:
+            Authenticate as a service principal using a client secret to access a source blob. Ensure "bearer " is
+            the prefix of the source_authorization string.
         """
         options = self._upload_range_from_url_options(
             source_url=source_url,
@@ -1091,7 +1106,7 @@ class ShareFileClient(StorageAccountHostsMixin):
         )
         try:
             return self._client.file.upload_range_from_url(**options)  # type: ignore
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     def _get_ranges_options( # type: ignore
@@ -1161,7 +1176,7 @@ class ShareFileClient(StorageAccountHostsMixin):
             **kwargs)
         try:
             ranges = self._client.file.get_range_list(**options)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         return [{'start': file_range.start, 'end': file_range.end} for file_range in ranges.ranges]
 
@@ -1204,7 +1219,7 @@ class ShareFileClient(StorageAccountHostsMixin):
             **kwargs)
         try:
             ranges = self._client.file.get_range_list(**options)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         return get_file_ranges_result(ranges)
 
@@ -1252,11 +1267,12 @@ class ShareFileClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=return_response_headers,
                 content_length=0,
+                optionalbody=None,
                 file_range_write="clear",
                 range=content_range,
                 lease_access_conditions=access_conditions,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -1291,7 +1307,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 cls=return_response_headers,
                 timeout=timeout,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -1348,7 +1364,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                 'closed_handles_count': response.get('number_of_handles_closed', 0),
                 'failed_handles_count': response.get('number_of_handles_failed', 0)
             }
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -1381,7 +1397,7 @@ class ShareFileClient(StorageAccountHostsMixin):
                     cls=return_response_headers,
                     **kwargs
                 )
-            except StorageErrorException as error:
+            except HttpResponseError as error:
                 process_storage_error(error)
             continuation_token = response.get('marker')
             try_close = bool(continuation_token)
