@@ -16,6 +16,7 @@ except ImportError:
 
 import six
 
+from azure.core.exceptions import HttpResponseError
 from azure.core.paging import ItemPaged
 from azure.core.tracing.decorator import distributed_trace
 from ._shared.base_client import StorageAccountHostsMixin, parse_connection_str, parse_query
@@ -26,8 +27,8 @@ from ._shared.response_handlers import (
     return_headers_and_deserialized)
 from ._message_encoding import NoEncodePolicy, NoDecodePolicy
 from ._deserialize import deserialize_queue_properties, deserialize_queue_creation
-from ._generated import AzureQueueStorage, VERSION
-from ._generated.models import StorageErrorException, SignedIdentifier
+from ._generated import AzureQueueStorage
+from ._generated.models import SignedIdentifier
 from ._generated.models import QueueMessage as GenQueueMessage
 from ._models import QueueMessage, AccessPolicy, MessagesPaged
 
@@ -40,6 +41,10 @@ if TYPE_CHECKING:
 class QueueClient(StorageAccountHostsMixin):
     """A client to interact with a specific Queue.
 
+    For more optional configuration, please click
+    `here <https://github.com/Azure/azure-sdk-for-python/tree/master/sdk/storage/azure-storage-queue
+    #optional-configuration>`_.
+
     :param str account_url:
         The URL to the storage account. In order to create a client given the full URI to the queue,
         use the :func:`from_queue_url` classmethod.
@@ -47,7 +52,8 @@ class QueueClient(StorageAccountHostsMixin):
     :type queue_name: str
     :param credential:
         The credentials with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string, an account
+        account URL already has a SAS token. The value can be a SAS token string,
+        an instance of a AzureSasCredential from azure.core.credentials, an account
         shared access key, or an instance of a TokenCredentials class from azure.identity.
     :keyword str api_version:
         The Storage API version to use for requests. Default value is '2019-07-07'.
@@ -99,7 +105,8 @@ class QueueClient(StorageAccountHostsMixin):
         self._config.message_encode_policy = kwargs.get('message_encode_policy', None) or NoEncodePolicy()
         self._config.message_decode_policy = kwargs.get('message_decode_policy', None) or NoDecodePolicy()
         self._client = AzureQueueStorage(self.url, pipeline=self._pipeline)
-        self._client._config.version = kwargs.get('api_version', VERSION)  # pylint: disable=protected-access
+        default_api_version = self._client._config.version  # pylint: disable=protected-access
+        self._client._config.version = kwargs.get('api_version', default_api_version)  # pylint: disable=protected-access
 
     def _format_url(self, hostname):
         """Format the endpoint URL according to the current location
@@ -122,7 +129,8 @@ class QueueClient(StorageAccountHostsMixin):
         :param str queue_url: The full URI to the queue, including SAS token if used.
         :param credential:
             The credentials with which to authenticate. This is optional if the
-            account URL already has a SAS token. The value can be a SAS token string, an account
+            account URL already has a SAS token. The value can be a SAS token string,
+            an instance of a AzureSasCredential from azure.core.credentials, an account
             shared access key, or an instance of a TokenCredentials class from azure.identity.
         :returns: A queue client.
         :rtype: ~azure.storage.queue.QueueClient
@@ -168,7 +176,8 @@ class QueueClient(StorageAccountHostsMixin):
         :param credential:
             The credentials with which to authenticate. This is optional if the
             account URL already has a SAS token, or the connection string already has shared
-            access key values. The value can be a SAS token string, an account shared access
+            access key values. The value can be a SAS token string,
+            an instance of a AzureSasCredential from azure.core.credentials, an account shared access
             key, or an instance of a TokenCredentials class from azure.identity.
         :returns: A queue client.
         :rtype: ~azure.storage.queue.QueueClient
@@ -226,7 +235,7 @@ class QueueClient(StorageAccountHostsMixin):
                 headers=headers,
                 cls=deserialize_queue_creation,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -258,7 +267,7 @@ class QueueClient(StorageAccountHostsMixin):
         timeout = kwargs.pop('timeout', None)
         try:
             self._client.queue.delete(timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -288,7 +297,7 @@ class QueueClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=deserialize_queue_properties,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         response.name = self.queue_name
         return response # type: ignore
@@ -325,7 +334,7 @@ class QueueClient(StorageAccountHostsMixin):
                 headers=headers,
                 cls=return_response_headers,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -345,7 +354,7 @@ class QueueClient(StorageAccountHostsMixin):
                 timeout=timeout,
                 cls=return_headers_and_deserialized,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
         return {s.id: s.access_policy or AccessPolicy() for s in identifiers}
 
@@ -400,7 +409,7 @@ class QueueClient(StorageAccountHostsMixin):
                 queue_acl=signed_identifiers or None,
                 timeout=timeout,
                 **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -478,7 +487,63 @@ class QueueClient(StorageAccountHostsMixin):
             queue_message.pop_receipt = enqueued[0].pop_receipt
             queue_message.next_visible_on = enqueued[0].time_next_visible
             return queue_message
-        except StorageErrorException as error:
+        except HttpResponseError as error:
+            process_storage_error(error)
+
+    @distributed_trace
+    def receive_message(self, **kwargs):
+        # type: (Optional[Any]) -> QueueMessage
+        """Removes one message from the front of the queue.
+
+        When the message is retrieved from the queue, the response includes the message
+        content and a pop_receipt value, which is required to delete the message.
+        The message is not automatically deleted from the queue, but after it has
+        been retrieved, it is not visible to other clients for the time interval
+        specified by the visibility_timeout parameter.
+
+        If the key-encryption-key or resolver field is set on the local service object, the message will be
+        decrypted before being returned.
+
+        :keyword int visibility_timeout:
+            If not specified, the default value is 0. Specifies the
+            new visibility timeout value, in seconds, relative to server time.
+            The value must be larger than or equal to 0, and cannot be
+            larger than 7 days. The visibility timeout of a message cannot be
+            set to a value later than the expiry time. visibility_timeout
+            should be set to a value smaller than the time-to-live value.
+        :keyword int timeout:
+            The server timeout, expressed in seconds.
+        :return:
+            Returns a message from the Queue.
+        :rtype: ~azure.storage.queue.QueueMessage
+
+        .. admonition:: Example:
+
+            .. literalinclude:: ../samples/queue_samples_message.py
+                :start-after: [START receive_one_message]
+                :end-before: [END receive_one_message]
+                :language: python
+                :dedent: 12
+                :caption: Receive one message from the queue.
+        """
+        visibility_timeout = kwargs.pop('visibility_timeout', None)
+        timeout = kwargs.pop('timeout', None)
+        self._config.message_decode_policy.configure(
+            require_encryption=self.require_encryption,
+            key_encryption_key=self.key_encryption_key,
+            resolver=self.key_resolver_function)
+        try:
+            message = self._client.messages.dequeue(
+                number_of_messages=1,
+                visibilitytimeout=visibility_timeout,
+                timeout=timeout,
+                cls=self._config.message_decode_policy,
+                **kwargs
+            )
+            wrapped_message = QueueMessage._from_generated(  # pylint: disable=protected-access
+                message[0]) if message != [] else None
+            return wrapped_message
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -500,6 +565,17 @@ class QueueClient(StorageAccountHostsMixin):
             messages to retrieve from the queue, up to a maximum of 32. If
             fewer are visible, the visible messages are returned. By default,
             a single message is retrieved from the queue with this operation.
+            `by_page()` can be used to provide a page iterator on the AsyncItemPaged if messages_per_page is set.
+            `next()` can be used to get the next page.
+        .. admonition:: Example:
+
+            .. literalinclude:: ../samples/queue_samples_message.py
+                :start-after: [START receive_messages_listing]
+                :end-before: [END receive_messages_listing]
+                :language: python
+                :dedent: 12
+                :caption: List pages and corresponding messages from the queue.
+
         :keyword int visibility_timeout:
             If not specified, the default value is 0. Specifies the
             new visibility timeout value, in seconds, relative to server time.
@@ -538,7 +614,7 @@ class QueueClient(StorageAccountHostsMixin):
                 **kwargs
             )
             return ItemPaged(command, results_per_page=messages_per_page, page_iterator_class=MessagesPaged)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -635,7 +711,7 @@ class QueueClient(StorageAccountHostsMixin):
             new_message.pop_receipt = response['popreceipt']
             new_message.next_visible_on = response['time_next_visible']
             return new_message
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -693,7 +769,7 @@ class QueueClient(StorageAccountHostsMixin):
             for peeked in messages:
                 wrapped_messages.append(QueueMessage._from_generated(peeked))  # pylint: disable=protected-access
             return wrapped_messages
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -716,7 +792,7 @@ class QueueClient(StorageAccountHostsMixin):
         timeout = kwargs.pop('timeout', None)
         try:
             self._client.messages.clear(timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -769,5 +845,5 @@ class QueueClient(StorageAccountHostsMixin):
                 queue_message_id=message_id,
                 **kwargs
             )
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
